# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, grpc_helpers_async, operations_v1
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
import grpc  # type: ignore
from grpc.experimental import aio  # type: ignore

from google.cloud.gke_multicloud_v1.types import aws_resources, aws_service

from .base import DEFAULT_CLIENT_INFO, AwsClustersTransport
from .grpc import AwsClustersGrpcTransport


class AwsClustersGrpcAsyncIOTransport(AwsClustersTransport):
    """gRPC AsyncIO backend transport for AwsClusters.

    The AwsClusters API provides a single centrally managed
    service to create and manage Anthos clusters that run on AWS
    infrastructure.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "gkemulticloud.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "gkemulticloud.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: Optional[aio.Channel] = None,
        api_mtls_endpoint: Optional[str] = None,
        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[aio.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}
        self._operations_client: Optional[operations_v1.OperationsAsyncClient] = None

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def operations_client(self) -> operations_v1.OperationsAsyncClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Quick check: Only create a new client if we do not already have one.
        if self._operations_client is None:
            self._operations_client = operations_v1.OperationsAsyncClient(
                self.grpc_channel
            )

        # Return the client from cache.
        return self._operations_client

    @property
    def create_aws_cluster(
        self,
    ) -> Callable[
        [aws_service.CreateAwsClusterRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the create aws cluster method over gRPC.

        Creates a new
        [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster] resource
        on a given GCP project and region.

        If successful, the response contains a newly created
        [Operation][google.longrunning.Operation] resource that can be
        described to track the status of the operation.

        Returns:
            Callable[[~.CreateAwsClusterRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_aws_cluster" not in self._stubs:
            self._stubs["create_aws_cluster"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/CreateAwsCluster",
                request_serializer=aws_service.CreateAwsClusterRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_aws_cluster"]

    @property
    def update_aws_cluster(
        self,
    ) -> Callable[
        [aws_service.UpdateAwsClusterRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the update aws cluster method over gRPC.

        Updates an
        [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster].

        Returns:
            Callable[[~.UpdateAwsClusterRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_aws_cluster" not in self._stubs:
            self._stubs["update_aws_cluster"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/UpdateAwsCluster",
                request_serializer=aws_service.UpdateAwsClusterRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["update_aws_cluster"]

    @property
    def get_aws_cluster(
        self,
    ) -> Callable[
        [aws_service.GetAwsClusterRequest], Awaitable[aws_resources.AwsCluster]
    ]:
        r"""Return a callable for the get aws cluster method over gRPC.

        Describes a specific
        [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster] resource.

        Returns:
            Callable[[~.GetAwsClusterRequest],
                    Awaitable[~.AwsCluster]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_aws_cluster" not in self._stubs:
            self._stubs["get_aws_cluster"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/GetAwsCluster",
                request_serializer=aws_service.GetAwsClusterRequest.serialize,
                response_deserializer=aws_resources.AwsCluster.deserialize,
            )
        return self._stubs["get_aws_cluster"]

    @property
    def list_aws_clusters(
        self,
    ) -> Callable[
        [aws_service.ListAwsClustersRequest],
        Awaitable[aws_service.ListAwsClustersResponse],
    ]:
        r"""Return a callable for the list aws clusters method over gRPC.

        Lists all [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster]
        resources on a given Google Cloud project and region.

        Returns:
            Callable[[~.ListAwsClustersRequest],
                    Awaitable[~.ListAwsClustersResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_aws_clusters" not in self._stubs:
            self._stubs["list_aws_clusters"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/ListAwsClusters",
                request_serializer=aws_service.ListAwsClustersRequest.serialize,
                response_deserializer=aws_service.ListAwsClustersResponse.deserialize,
            )
        return self._stubs["list_aws_clusters"]

    @property
    def delete_aws_cluster(
        self,
    ) -> Callable[
        [aws_service.DeleteAwsClusterRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the delete aws cluster method over gRPC.

        Deletes a specific
        [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster] resource.

        Fails if the cluster has one or more associated
        [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool]
        resources.

        If successful, the response contains a newly created
        [Operation][google.longrunning.Operation] resource that can be
        described to track the status of the operation.

        Returns:
            Callable[[~.DeleteAwsClusterRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_aws_cluster" not in self._stubs:
            self._stubs["delete_aws_cluster"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/DeleteAwsCluster",
                request_serializer=aws_service.DeleteAwsClusterRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["delete_aws_cluster"]

    @property
    def generate_aws_access_token(
        self,
    ) -> Callable[
        [aws_service.GenerateAwsAccessTokenRequest],
        Awaitable[aws_service.GenerateAwsAccessTokenResponse],
    ]:
        r"""Return a callable for the generate aws access token method over gRPC.

        Generates a short-lived access token to authenticate to a given
        [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster] resource.

        Returns:
            Callable[[~.GenerateAwsAccessTokenRequest],
                    Awaitable[~.GenerateAwsAccessTokenResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "generate_aws_access_token" not in self._stubs:
            self._stubs["generate_aws_access_token"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/GenerateAwsAccessToken",
                request_serializer=aws_service.GenerateAwsAccessTokenRequest.serialize,
                response_deserializer=aws_service.GenerateAwsAccessTokenResponse.deserialize,
            )
        return self._stubs["generate_aws_access_token"]

    @property
    def create_aws_node_pool(
        self,
    ) -> Callable[
        [aws_service.CreateAwsNodePoolRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the create aws node pool method over gRPC.

        Creates a new
        [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool],
        attached to a given
        [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster].

        If successful, the response contains a newly created
        [Operation][google.longrunning.Operation] resource that can be
        described to track the status of the operation.

        Returns:
            Callable[[~.CreateAwsNodePoolRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_aws_node_pool" not in self._stubs:
            self._stubs["create_aws_node_pool"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/CreateAwsNodePool",
                request_serializer=aws_service.CreateAwsNodePoolRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_aws_node_pool"]

    @property
    def update_aws_node_pool(
        self,
    ) -> Callable[
        [aws_service.UpdateAwsNodePoolRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the update aws node pool method over gRPC.

        Updates an
        [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool].

        Returns:
            Callable[[~.UpdateAwsNodePoolRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_aws_node_pool" not in self._stubs:
            self._stubs["update_aws_node_pool"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/UpdateAwsNodePool",
                request_serializer=aws_service.UpdateAwsNodePoolRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["update_aws_node_pool"]

    @property
    def get_aws_node_pool(
        self,
    ) -> Callable[
        [aws_service.GetAwsNodePoolRequest], Awaitable[aws_resources.AwsNodePool]
    ]:
        r"""Return a callable for the get aws node pool method over gRPC.

        Describes a specific
        [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool]
        resource.

        Returns:
            Callable[[~.GetAwsNodePoolRequest],
                    Awaitable[~.AwsNodePool]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_aws_node_pool" not in self._stubs:
            self._stubs["get_aws_node_pool"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/GetAwsNodePool",
                request_serializer=aws_service.GetAwsNodePoolRequest.serialize,
                response_deserializer=aws_resources.AwsNodePool.deserialize,
            )
        return self._stubs["get_aws_node_pool"]

    @property
    def list_aws_node_pools(
        self,
    ) -> Callable[
        [aws_service.ListAwsNodePoolsRequest],
        Awaitable[aws_service.ListAwsNodePoolsResponse],
    ]:
        r"""Return a callable for the list aws node pools method over gRPC.

        Lists all
        [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool]
        resources on a given
        [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster].

        Returns:
            Callable[[~.ListAwsNodePoolsRequest],
                    Awaitable[~.ListAwsNodePoolsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_aws_node_pools" not in self._stubs:
            self._stubs["list_aws_node_pools"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/ListAwsNodePools",
                request_serializer=aws_service.ListAwsNodePoolsRequest.serialize,
                response_deserializer=aws_service.ListAwsNodePoolsResponse.deserialize,
            )
        return self._stubs["list_aws_node_pools"]

    @property
    def delete_aws_node_pool(
        self,
    ) -> Callable[
        [aws_service.DeleteAwsNodePoolRequest], Awaitable[operations_pb2.Operation]
    ]:
        r"""Return a callable for the delete aws node pool method over gRPC.

        Deletes a specific
        [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool]
        resource.

        If successful, the response contains a newly created
        [Operation][google.longrunning.Operation] resource that can be
        described to track the status of the operation.

        Returns:
            Callable[[~.DeleteAwsNodePoolRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_aws_node_pool" not in self._stubs:
            self._stubs["delete_aws_node_pool"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/DeleteAwsNodePool",
                request_serializer=aws_service.DeleteAwsNodePoolRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["delete_aws_node_pool"]

    @property
    def get_aws_server_config(
        self,
    ) -> Callable[
        [aws_service.GetAwsServerConfigRequest],
        Awaitable[aws_resources.AwsServerConfig],
    ]:
        r"""Return a callable for the get aws server config method over gRPC.

        Returns information, such as supported AWS regions
        and Kubernetes versions, on a given Google Cloud
        location.

        Returns:
            Callable[[~.GetAwsServerConfigRequest],
                    Awaitable[~.AwsServerConfig]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_aws_server_config" not in self._stubs:
            self._stubs["get_aws_server_config"] = self.grpc_channel.unary_unary(
                "/google.cloud.gkemulticloud.v1.AwsClusters/GetAwsServerConfig",
                request_serializer=aws_service.GetAwsServerConfigRequest.serialize,
                response_deserializer=aws_resources.AwsServerConfig.deserialize,
            )
        return self._stubs["get_aws_server_config"]

    def close(self):
        return self.grpc_channel.close()


__all__ = ("AwsClustersGrpcAsyncIOTransport",)
